import sendgrid
from sendgrid.helpers.mail import *
import random

try:
    with open("key", 'r') as fh:
        keys=fh.read()
except FileNotFoundError as e:
    with open("key", "w") as fh:
        KEY=str(input("Paste Key:  "))
        fh.write(KEY)
        keys=KEY


def send(from_email, to_email, subject, body):
    sg = sendgrid.SendGridAPIClient(apikey=keys)
    from_email = Email(from_email)
    to_email = Email(to_email)
    content = Content('text/plain', body)
    mail = Mail(from_email, subject, to_email, content)
    response = sg.client.mail.send.post(request_body=mail.get())
    print('sent email')
    print(response.status_code)
    return response



#model
import time
import random
from platform import uname
from loguru import logger as log # TODO: Configure proper logger
import io
import queue
import sendgrid
import threading


from . import engineapi
# TODO: Create Stats Model
k=open("mailing_list.xo", "a")
k.close()
FROM = uname().node+"@mylli.supercode.xo"
# TODO: Create Custom Queue class
class sMail(object):
    def __init__(self, mFrom, to, subject, body,html_body=None):
        self.mFrom = mFrom
        self.to = to
        self.subject = subject
        self.body = body

    def __call__(self):
        #return tuple
        return (self.mFrom,  self.subject,self.to, self.body)


class MailingList(object):

    def __init__(self, mails=None, mFrom=FROM, api_key=None):
        super(MailingList, self).__init__()
        self.mails = mails
        self.delay = 5
        self.KEY = None

        self.mail_queue = queue.Queue(100) #maximum Sendgrid limit
        #load date

        #Mail Objects
        # TODO: Load mail objects from COnstrctor

    def add_addr_to_list(self, addrs):
        #Save email address to file for mailing list
        with io.open("mailing_list.xo", "a+") as fh:
            for addr in addrs:
                if addr is "invalid": continue
                fh.write(addr+"\n")
                log.debug("Added {} to Mailing List".format(addr))
                # TODO: Show entry index

    @staticmethod
    def load_addresses(mailing_list="mailing_list.xo"):
        #load addresses from file
        with open(mailing_list, 'r') as fh:
            addrs = fh.read()

            return addrs.split("\n")
    def get_mailing_list(self):
        flist=""
        emails = self.load_addresses()
        for i in emails:
            flist+=str(emails.index(i)) +". " +i+"\n"
        return flist

    def broadcast(self, subject,  body, mFrom=FROM,template=None):
        #broadcast to mailing list
        #load addrs
        recipients = self.load_addresses()
        if len(recipients) > 100:
            log.warning("More than 100 emails detected")
        for rec in recipients:
            if not self.mail_queue.full():
                self.mail_queue.put(sMail(mFrom, rec, subject, body )) # TODO: Allow templates
        self.clear_queue()
        log.info("broadcast Complete")

    def remove_address(self, q=None):
        # TODO: Fix this
        new_addrs =[]
        if q is not None:
            with io.open("mailing_list.xo", "r+") as fh:
                addrs=fh.read()
                fh.truncate() #sorry...
                # print(addrs.split())
                for i in addrs.split():
                    uname = i.split("@")[0]
                    # print(i)
                    # TODO: Can do better here
                    if not uname.startswith(q):
                        new_addrs.append(i)
                fh.close()

        self.add_addr_to_list(new_addrs)


    def start_jb(self):
        t=threading.Thread(target=self.clear_queue, name="Mail Queue")
        t.start()
        log.debug("Mail Queue Started")



    def send_email(self, mail):
        #sendgrid
        # TODO: Refactor the whole class and inherit from sendgrid
        self.mail_queue.put(mail)
        self.start_jb()
        # self.add_addr_to_list(mail)
        # log.info("enQueued ")


    def clear_queue(self):

        while not self.mail_queue.empty():
            #sendgrid adn send
            # TODO: Error Handling
            mail_package = self.mail_queue.get()
            res =engineapi.send(mail_package.mFrom, mail_package.to, mail_package.subject, mail_package.body)
            log.debug(res.status_code)

    @staticmethod
    def set_key(lkey):
        with io.open("key", "w") as fh:
            fh.write(lkey)


test = MailingList()



import sys
import os
import argparse
from .model import MailingList, sMail
from loguru import logger as log
from platform import uname
import random
from pyfiglet import Figlet
# import sendgrid
#Commands

# add -> add address to mailing list
# broadcast -> broadcast message
# mailist -> return addresses in list
# send -> invoke sending prompt
# stats -> return stats
figFonts = Figlet().getFonts()



parser = argparse.ArgumentParser()

parser.add_argument("--mFrom" , help="Sender Address", default=uname().node+"@mylli.supercode.xo")
parser.add_argument("--to", help="destination address", default="ilovebugsincode@gmail.com" )
parser.add_argument("-s", "--subject", help="Subject of Email", default="I love mylli")
parser.add_argument("-b", "--body", help="Email Body", default="mylli works well")
parser.add_argument("-t", "--template", help="Load Template", default=None)
parser.add_argument("-a", "--add", help="Add Address to Mailing List")
# margs = parser.parse_args()
#Databse Engine to store emails
mylli = MailingList()





def main():
    # Add email to mail cache
    # Prepare email
    args = sys.argv[1:]
    if len(args)==0:
        f=Figlet(random.choice(figFonts))
        print(f.renderText("mylli"))
        print("Simple Mailer by Gishobert (SuperCode) Gwenzi")

    elif args[0] == "add":
        #
        email_addr = args[1]
        mylli.add_addr_to_list([email_addr])
        log.info("Succeessfully Added to MailingList")

        # TODO: regexmto validate email



        #add email addres to mailing list
    elif args[0] == "broadcast":
        #broadcast to addrresses in list
        #subject
        # from
        #body
        # TODO: Parse args properly
        subject, mFrom, body = args[1:]
        mylli.broadcast(subject, mFrom, body)

    elif args[0] == "configure":
        KEY=str(input("Sendgrid Key"))
        mylli.set_key(KEY)

    elif args[0] == "mailist":
        #return numbered list of mails in list
        print(mylli.get_mailing_list())
    elif args[0] == "send":
        #start email sending prompt
        subject = str(input("Subject: "))
        mFrom = str(input("From: "))
        body = str(input("Body: ")) # TODO: Allo loading from template
        recipients = str(input("Recipients, separate with comma: "))

        for to in recipients.split(","):
            mail_obj = sMail(mFrom,to, subject, body)
            mylli.send_email(mail_obj)
    elif args[0] == "stats":
        #return stats
        pass
    elif args[0] == "blacklist":
        #blacklist addr from list
        pass

    else:
        f=Figlet(random.choice(figFonts))
        print(f.renderText("mylli"))
        print("Bye!!!!")

if __name__ == '__main__':
    main()



from setuptools import setup
import setuptools
with open("README.md", 'r') as fh:
    long_description = fh.read()

__version__ = "0.2.0"

setup(name="mylli",
	version="0.2.0",
	description="Simple Mailing Client",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/ignertic/mylli",
	author="Gishobert (SuperCode) Gwenzi",
	author_email="ilovebugsincode@gmail.com",
	licence="MIT",
	classifiers=[
		"Programming Language :: Python :: 3",
		"Operating System :: OS Independent"],
	packages=setuptools.find_packages(),
	include_package_data=True,
	install_requires=["requests","sendgrid", "loguru" ],
	entry_points={"console_scripts" : ["mylli=mylli.__main__:main" ]},
	) # TODO: Add more console scripts
